﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms.DataVisualization.Charting;
using HeuristicLab.MainForm;

namespace HeuristicLab.Problems.DataAnalysis.Views.Regression {
  [View("Residual Histogram")]
  [Content(typeof(IRegressionSolution))]
  public partial class RegressionSolutionResidualHistogram : DataAnalysisSolutionEvaluationView {
    private const string ALL_SERIES = "All samples";
    private const string TRAINING_SERIES = "Training samples";
    private const string TEST_SERIES = "Test samples";

    // for test purposes
    private const int bins = 9;

    public new IRegressionSolution Content {
      get { return (IRegressionSolution)base.Content; }
      set { base.Content = value; }
    }

    public RegressionSolutionResidualHistogram() {
      InitializeComponent();

      this.chart.Series.Add(ALL_SERIES);
      this.chart.Series[ALL_SERIES].LegendText = ALL_SERIES;
      this.chart.Series[ALL_SERIES].ChartType = SeriesChartType.Column;

      //this.chart.Series.Add(TRAINING_SERIES);
      //this.chart.Series[TRAINING_SERIES].LegendText = TRAINING_SERIES;
      //this.chart.Series[TRAINING_SERIES].ChartType = SeriesChartType.FastPoint;
      //this.chart.Series[TRAINING_SERIES].Points.Add(1.0);

      //this.chart.Series.Add(TEST_SERIES);
      //this.chart.Series[TEST_SERIES].LegendText = TEST_SERIES;
      //this.chart.Series[TEST_SERIES].ChartType = SeriesChartType.FastPoint;

      //configure axis 
      this.chart.CustomizeAllChartAreas();
      this.chart.ChartAreas[0].AxisX.Title = "Residuals";
      this.chart.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
      this.chart.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
      this.chart.ChartAreas[0].CursorX.Interval = 1;
      this.chart.ChartAreas[0].CursorY.Interval = 1;

      this.chart.ChartAreas[0].AxisY.Title = "Relative Frequency";
      this.chart.ChartAreas[0].CursorY.IsUserSelectionEnabled = true;
      this.chart.ChartAreas[0].AxisY.ScaleView.Zoomable = true;
      this.chart.ChartAreas[0].AxisY.IsStartedFromZero = true;
    }

    private void RedrawChart() {
      this.chart.Series[ALL_SERIES].Points.Clear();
      if (Content != null) {
        List<double> residuals = new List<double>();
        IEnumerable<double> targetValues = Content.ProblemData.Dataset.GetDoubleValues(Content.ProblemData.TargetVariable);
        IEnumerable<double> estimatedValues = Content.EstimatedValues;
        for (int i = 0; i < Content.ProblemData.Dataset.Rows; i++) {
          residuals.Add(estimatedValues.ElementAt(i) - targetValues.ElementAt(i));
        }

        //no check if both are negative
        //assumption that all values are around zero
        double max = Math.Ceiling(Math.Max(Math.Abs(residuals.Min()), Math.Abs(residuals.Max())));
        double intervalWidth = (max * 2) / bins;
        double intervalCenter = intervalWidth / 2.0;
        double current = -max;

        Series histogramSeries = this.chart.Series[ALL_SERIES];

        Dictionary<double, int> frequencies = new Dictionary<double, int>();

        for (int i = 0; i <= bins; i++) {
          IEnumerable<double> help = residuals.Where(x => x >= (current - intervalCenter) && x < (current + intervalCenter));
          frequencies.Add(current, help.Count());
          current += intervalWidth;
        }

        foreach (var item in frequencies) {
          histogramSeries.Points.AddXY(item.Key, item.Value);
        }
      }
    }

    #region events
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ModelChanged += new EventHandler(Content_ModelChanged);
      Content.ProblemDataChanged += new EventHandler(Content_ProblemDataChanged);
    }
    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
      Content.ModelChanged -= new EventHandler(Content_ModelChanged);
      Content.ProblemDataChanged -= new EventHandler(Content_ProblemDataChanged);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      RedrawChart();
    }
    private void Content_ProblemDataChanged(object sender, EventArgs e) {
      RedrawChart();
    }
    private void Content_ModelChanged(object sender, EventArgs e) {
      RedrawChart();
    }
    #endregion
  }
}
